<script type="text/javascript">
    var Cufon = Cufon || null;
    if(Cufon) Cufon.now();

    var _gaq = _gaq || [];
    {% if analytics_id %}
        _gaq.push(['_setAccount', '{{ analytics_id }}']);
        {% if trackPageview %}
        _gaq.push(['_trackPageview']);
        {% endif %}
        var order_success = order_success || null;
        if(order_success) order_transaction();

        (function() {
            var ga = document.createElement('script'); ga.type = 'text/javascript'; ga.async = true;
            ga.src = ('https:' == document.location.protocol ? 'https://' : 'http://') + 'stats.g.doubleclick.net/dc.js';
            var s = document.getElementsByTagName('script')[0]; s.parentNode.insertBefore(ga, s);
        })();
    {% endif %}
</script>
