{% load i18n %}
<form action="/mailchimp/" methode="post" id="mailchimp-form" role="form">
    {% csrf_token %}
    <div class="input-group large">
        <input name="email" id="id_email" type="email" placeholder="{% trans "Ihre E-Mail Adresse" %}" class="form-control" value="{{ mailchimp_signup_form.email.value|default:"" }}">
        <input name="language" id="id_language" type="hidden" value="{{ feincms_page.language }}">
        <span class="input-group-btn">
            <button type="submit" class="btn btn-default">{% trans "Anmelden" %}</button>
        </span>
    </div>
    <div class="text-left error-msg-box">
        {{mailchimp_signup_form.email.errors}}
    </div>
</form>
